<?php

namespace Drupal\gotem\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configuration form for the Gotem module.
 */
class GotemSettingsForm extends ConfigFormBase {

  /**
   * The entity field manager service.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * Constructs a new GotemSettingsForm.
   *
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   *   The entity field manager service.
   */
  public function __construct(EntityFieldManagerInterface $entity_field_manager) {
    $this->entityFieldManager = $entity_field_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_field.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['gotem.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'gotem_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('gotem.settings');

    // API Key Field
    $form['api_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('OpenAI API Key'),
      '#default_value' => $config->get('api_key'),
      '#required' => TRUE,
      '#description' => $this->t('Enter your OpenAI API key here.'),
      '#maxlength' => 256, // Increased to accommodate longer keys
      '#size' => 64,        // Adjusted for better visibility
    ];

    // Media Reference Field Selection
    $form['media_reference_field'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Media Reference Field'),
      '#default_value' => $config->get('media_reference_field') ?: 'field_media',
      '#required' => TRUE,
      '#description' => $this->t('Enter the machine name of the media reference field (e.g., field_media).'),
      '#maxlength' => 128,
      '#size' => 32,
    ];

    // Moderated Fields Selection for Content Entities
    $form['moderated_fields'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Fields to Moderate (Content Entities)'),
      '#description' => $this->t('Select the fields you want to moderate using the OpenAI Moderation API.'),
      '#options' => $this->getModeratableFields('node'), // Assuming 'node' entity type
      '#default_value' => $config->get('moderated_fields') ?: [],
      '#required' => TRUE,
    ];

    // Moderated Fields Selection for Media Entities
    $form['moderated_fields_media'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Fields to Moderate (Media Entities)'),
      '#description' => $this->t('Select the fields you want to moderate in media entities using the OpenAI Moderation API.'),
      '#options' => $this->getModeratableFields('media'), // 'media' entity type
      '#default_value' => $config->get('moderated_fields_media') ?: [],
      '#required' => TRUE,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * Retrieves a list of fields eligible for moderation for a given entity type.
   *
   * @param string $entity_type
   *   The entity type ('node' or 'media').
   *
   * @return array
   *   An associative array of field machine names and their labels.
   */
  protected function getModeratableFields($entity_type) {
    $fields = [];

    // Retrieve all bundles for the given entity type.
    $bundles = \Drupal::entityTypeManager()->getStorage($entity_type)->loadMultiple();

    foreach ($bundles as $bundle_name => $bundle) {
      $field_definitions = $this->entityFieldManager->getFieldDefinitions($entity_type, $bundle_name);

      foreach ($field_definitions as $field_name => $field_definition) {
        $field_type = $field_definition->getType();

        // Target specific field types: 'string', 'string_long', 'text', 'text_long'
        if (in_array($field_type, ['string', 'string_long', 'text', 'text_long'])) {
          // Construct a label that includes the bundle name for clarity.
          $label = $field_definition->getLabel() . ' (' . $bundle->label() . ')';
          $fields[$field_name] = $label;
        }
      }
    }

    return $fields;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // Ensure at least one field is selected for content entities.
    $selected_fields = array_filter($form_state->getValue('moderated_fields'));
    if (empty($selected_fields)) {
      $form_state->setErrorByName('moderated_fields', $this->t('You must select at least one field to moderate for content entities.'));
    }

    // Ensure at least one field is selected for media entities.
    $selected_fields_media = array_filter($form_state->getValue('moderated_fields_media'));
    if (empty($selected_fields_media)) {
      $form_state->setErrorByName('moderated_fields_media', $this->t('You must select at least one field to moderate for media entities.'));
    }

    // Validate API Key Length (assuming API keys are at least 50 characters).
    $api_key = $form_state->getValue('api_key');
    if (strlen($api_key) < 50) {
      $form_state->setErrorByName('api_key', $this->t('The API key appears to be too short.'));
    }

    // Validate Media Reference Field Exists and is of type 'entity_reference'.
    $media_reference_field = $form_state->getValue('media_reference_field');
    if (!$media_reference_field) {
      $form_state->setErrorByName('media_reference_field', $this->t('The media reference field cannot be empty.'));
    }
    else {
      // Check if the field exists on the specified entity type (assuming 'node' for content entities).
      // Adjust as needed based on your content types.
      $entity_type = 'node'; // Adjust if your media reference field is on a different entity type.
      $bundle = 'article'; // Replace with your content type bundle if different.

      $field_definitions = $this->entityFieldManager->getFieldDefinitions($entity_type, $bundle);
      if (!array_key_exists($media_reference_field, $field_definitions)) {
        $form_state->setErrorByName('media_reference_field', $this->t('The specified media reference field does not exist on the @bundle bundle of the @entity_type entity type.', [
          '@bundle' => $bundle,
          '@entity_type' => $entity_type,
        ]));
      }
      else {
        $field_definition = $field_definitions[$media_reference_field];
        if ($field_definition->getType() !== 'entity_reference') {
          $form_state->setErrorByName('media_reference_field', $this->t('The specified field is not an entity reference field.'));
        }
        else {
          // Further validate that it references media entities.
          $target_type = $field_definition->getSetting('target_type');
          if ($target_type !== 'media') {
            $form_state->setErrorByName('media_reference_field', $this->t('The specified entity reference field does not reference media entities.'));
          }
        }
      }
    }

    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Retrieve form values.
    $api_key = $form_state->getValue('api_key');
    $selected_fields = array_filter($form_state->getValue('moderated_fields'));
    $selected_fields_media = array_filter($form_state->getValue('moderated_fields_media'));
    $media_reference_field = $form_state->getValue('media_reference_field');

    // Save configuration.
    $this->config('gotem.settings')
      ->set('api_key', $api_key)
      ->set('moderated_fields', array_keys($selected_fields))
      ->set('moderated_fields_media', array_keys($selected_fields_media))
      ->set('media_reference_field', $media_reference_field)
      ->save();

    parent::submitForm($form, $form_state);
  }

}
